(*:Mathematica Version: 5.0.1 *)

(*:Package Version: 1.0.2 *)

(*:Name: Arrow3D`Arrow3D` *)

(*:Context:Arrow3D`Arrow3D` *)

(*:Title: Arrow3D`Arrow3D` *)

(*:Author: harry Calkins *)

(*:History:Created September 2000, 
 February,2002,Revised to allow arrows parallel
 to y-z plane. Added color options for the arrowhead.
 Modified 11Dec2003 to correct the case when arrow
 lies in x-zplane with slope -1.
 Modified March 2004 to correct for arrows lying in coordinate
 planes and added some options for arrow heads. Now all arrows
 in a plot can be rendered with heads of the same size.
*)

(*:Copyright: Copyright 2002,2004, Wolfram Research, Inc. *)

(*:Reference: Usage messages only. *)

(*:Summary:
This package provides Arrow functionality in three dimensions similar to 
the 2D package Graphics`Arrow`.
*)

(*:Keywords: Arrow3D *)

(*:Requirements: No special system requirements. *)


(*:Limitations:
None.
*)


BeginPackage["Arrow3D`Arrow3D`"];
	     
	     
Arrow3D::usage= "Arrow3D[point1, point2,opts] gives an arrow from \
point1 to point2 with an arrowhead of length .05 of the distance between the \
points unless a different value is specified for the HeadSize."

HeadColor::usage = "HeadColor is an option for Arrow3D which allows the user to
assign color to the arrow-head."

HeadSize::usage = "HeadSize is an option for Arrow3D which determines the length
of the arrow-head as a fraction of the length of the arrow. The default setting is .05."

UniformSize::usage ="A value for the option HeadSize which makes all the arrow heads the same size. 
UniformSize[aa] will cause all the arrow heads to be renedered aa units long."


Begin["`Private`"]

Options[Arrow3D] = {HeadColor -> GrayLevel[.3],HeadSize -> .05};

ArrowHead[pt1_,pt2_,opts___?OptionQ] := 
  Module[{a,b,c,aa,bb,cc,nrm1,nrm2,ttt,circ,pts,arrowpolys,hdcolor,hdsz,hdrad},
  
    hdcolor=HeadColor/.{opts}/.Options[Arrow3D];
    hdsz = HeadSize/.{opts}/.Options[Arrow3D];
    
    {aa,bb,cc} = pt2-pt1;
    
    (*Constructing an orthonormal basis for the arrowhead*)
    
    Which[aa == 0 && bb == 0, nrm1 = {0, 1, 0}; 
      nrm2 = {1, 0, 0}, aa == 0 && cc == 0, 
     nrm1 = {1, 0, 0}; nrm2 = {0, 0, 1}, 
     bb == 0 && cc == 0, nrm1 = {0, 0, 1}; 
      nrm2 = {0, 1, 0}, aa == 0, nrm1 = {1, 0, 0}; 
      nrm2 = Cross[nrm1, {0, bb, cc}/Norm[{bb, cc}]], 
     bb == 0, nrm1 = {0, 1, 0}; 
      nrm2 = Cross[nrm1, {aa, 0, cc}/Norm[{aa, cc}]], 
     cc == 0, nrm1 = {0, 0, 1}; 
      nrm2 = Cross[nrm1, {aa, bb, 0}/Norm[{aa, bb}]], 
     True, nrm1 = {-(((bb + cc)*Abs[aa])/
          (aa*Sqrt[2*aa^2 + (bb + cc)^2])), 
        Abs[aa]/Sqrt[2*aa^2 + (bb + cc)^2], 
        Abs[aa]/Sqrt[2*aa^2 + (bb + cc)^2]}; 
      nrm2 = {((bb - cc)*Abs[aa])/
   Sqrt[2*aa^4 + (bb + cc)^2*(bb^2 + cc^2) + 
     aa^2*(3*bb^2 + 2*bb*cc + 3*cc^2)], 
  -(((aa^2 + cc*(bb + cc))*Abs[aa])/
    (aa*Sqrt[2*aa^4 + (bb + cc)^2*(bb^2 + cc^2) + 
       aa^2*(3*bb^2 + 2*bb*cc + 3*cc^2)])), 
  ((aa^2 + bb*(bb + cc))*Abs[aa])/
   (aa*Sqrt[2*aa^4 + (bb + cc)^2*(bb^2 + cc^2) + 
      aa^2*(3*bb^2 + 2*bb*cc + 3*cc^2)])}];Which[aa == 0 && bb == 0, nrm1 = {0, 1, 0}; 
      nrm2 = {1, 0, 0}, aa == 0 && cc == 0, 
     nrm1 = {1, 0, 0}; nrm2 = {0, 0, 1}, 
     bb == 0 && cc == 0, nrm1 = {0, 0, 1}; 
      nrm2 = {0, 1, 0}, aa == 0, nrm1 = {1, 0, 0}; 
      nrm2 = Cross[nrm1, {0, bb, cc}/Norm[{bb, cc}]], 
     bb == 0, nrm1 = {0, 1, 0}; 
      nrm2 = Cross[nrm1, {aa, 0, cc}/Norm[{aa, cc}]], 
     cc == 0, nrm1 = {0, 0, 1}; 
      nrm2 = Cross[nrm1, {aa, bb, 0}/Norm[{aa, bb}]], 
     True, nrm1 = {-(((bb + cc)*Abs[aa])/
          (aa*Sqrt[2*aa^2 + (bb + cc)^2])), 
        Abs[aa]/Sqrt[2*aa^2 + (bb + cc)^2], 
        Abs[aa]/Sqrt[2*aa^2 + (bb + cc)^2]}; 
      nrm2 = {((bb - cc)*Abs[aa])/
   Sqrt[2*aa^4 + (bb + cc)^2*(bb^2 + cc^2) + 
     aa^2*(3*bb^2 + 2*bb*cc + 3*cc^2)], 
  -(((aa^2 + cc*(bb + cc))*Abs[aa])/
    (aa*Sqrt[2*aa^4 + (bb + cc)^2*(bb^2 + cc^2) + 
       aa^2*(3*bb^2 + 2*bb*cc + 3*cc^2)])), 
  ((aa^2 + bb*(bb + cc))*Abs[aa])/
   (aa*Sqrt[2*aa^4 + (bb + cc)^2*(bb^2 + cc^2) + 
      aa^2*(3*bb^2 + 2*bb*cc + 3*cc^2)])}];
    
    hdrad = If[Head[hdsz]===UniformSize,hdsz[[1]]/5,
      Sqrt[((pt2 -(hdsz pt1 + (1 - hdsz)pt2)).(pt2 -(hdsz pt1 + (1 - 
                            hdsz)pt2)))]/5];
    
    circ1[ttt_] := 
      hdrad nrm1 Cos[ttt]+ 
          hdrad nrm2 Sin[ttt]+(hdsz pt1 + (1 - hdsz)pt2);
           
    circ2[ttt_] := (hdvec = pt2+hdsz[[1]] (pt1-pt2)/Norm[(pt1-pt2)];
      hdrad nrm1 Cos[ttt]+ 
          hdrad nrm2 Sin[ttt]+hdvec);
           
    pts =If[Head[hdsz]===UniformSize,
        Partition[ N[Table[circ2[t],{t,0,2 Pi,Pi/5}]],2,1],
        Partition[ N[Table[circ1[t],{t,0,2 Pi,Pi/5}]],2,1]];               
    
    arrowpolys =
      Map[{EdgeForm[],SurfaceColor[hdcolor],
            Polygon[Flatten[{#,{pt2}},1]]}&,pts]
    ]
    
Arrow3D[pt1_,pt2_,opts___?OptionQ]:= If[Chop[pt2-pt1]=={0,0,0}, Point[pt1],
{Line[{pt1,pt2}],ArrowHead[pt1,pt2,opts]}]

End[];


Protect[Arrow3D];

EndPackage[]


